      ******************************************************************
      * Author: Erik Eriksen
      * Date: 04-19-2020
      * Updated: 03-12-2021
      * Purpose: Dice rolling application for DnD
      * Tectonics: cobc
      ******************************************************************
       IDENTIFICATION DIVISION.
       PROGRAM-ID. DND-DICE-ROLLER-PROGRAM.

       DATA DIVISION.

       FILE SECTION.

       WORKING-STORAGE SECTION.

       01  WS-DICE-INPUT.
           05  NUMBER-OF-DICE               PIC 9.
           05  FILLER                       PIC X.
           05  NUMBER-OF-SIDES              PIC 99.
           05  FILLER                       PIC X.
           05  VALUE-MODIFIER               PIC 99 VALUE ZERO.

       01  WS-DICE-OUTPUT.
           05  DICE-VALUE                   PIC 99.
           05  DICE-VALUE-MAX               PIC 99.
           05  DICE-VALUE-MAX-NO-MOD        PIC 99.
           05  DICE-VALUE-TOTAL             PIC 9(3).
           05  DICE-VALUE-TOTAL-NO-MOD      PIC 9(3).

       01  WS-USER-INPUT                    PIC X(10) VALUE SPACE.

       01  WS-TEMP-TIME                     PIC 9(9) VALUE ZERO.
       01  WS-UNUSED                        PIC 99.

       78  WS-NEW_LINE                      VALUE x"0a".

       PROCEDURE DIVISION.

      *>****************************************************************
      *>   Sets initial random seed value based on the time the
      *>   application was started. Follow up random calls will pull
      *>   next pseudo-random value in the sequence.
      *>****************************************************************
       INIT-RANDOM.
           ACCEPT WS-TEMP-TIME FROM TIME
           COMPUTE WS-UNUSED = FUNCTION RANDOM(WS-TEMP-TIME).

      *>****************************************************************
      *>   Entry point of application. Displays usage and handles user
      *>   input. Paragraphs and execution is performed based on values
      *>   entered by the user.
      *>****************************************************************
       MAIN-PROCEDURE.

           PERFORM PRINT-INSTRUCTIONS

           PERFORM UNTIL WS-USER-INPUT = "q" OR "Q"

               DISPLAY SPACE
               DISPLAY "Enter dice to roll: " WITH NO ADVANCING
               ACCEPT WS-USER-INPUT

               IF WS-USER-INPUT NOT EQUAL "q" AND "Q" THEN
                   IF WS-USER-INPUT(1:1) IS NUMERIC THEN
                       PERFORM COMPUTE-DICE-ROLL
                   ELSE
                       DISPLAY "Please enter a valid input."
                   END-IF
               END-IF

           END-PERFORM

           DISPLAY "Goodbye."
           STOP RUN.


      *>****************************************************************
      *>   Uses user input to create dice input. Then the dice will be
      *>   rolled for the number of times entered.
      *>   Totals are displayed after rolls are completed.
      *>****************************************************************
       COMPUTE-DICE-ROLL.
           MOVE WS-USER-INPUT TO WS-DICE-INPUT
           MOVE ZEROS TO WS-DICE-OUTPUT

           COMPUTE NUMBER-OF-SIDES = FUNCTION NUMVAL(NUMBER-OF-SIDES)

           COMPUTE VALUE-MODIFIER = FUNCTION NUMVAL(VALUE-MODIFIER)

           DISPLAY "Number of dice: " NUMBER-OF-DICE
           DISPLAY "Number of sides: " NUMBER-OF-SIDES
           DISPLAY "Modifier: " VALUE-MODIFIER

           PERFORM ROLL-DICE NUMBER-OF-DICE TIMES

           DISPLAY SPACE
           DISPLAY "Max roll: " DICE-VALUE-MAX-NO-MOD
           DISPLAY "Roll total: " DICE-VALUE-TOTAL-NO-MOD

           IF VALUE-MODIFIER NOT ZERO THEN
               DISPLAY
                   "Max roll (with modifier): " DICE-VALUE-MAX
               END-DISPLAY
               DISPLAY
                   "Roll total (with modifier): "
                   DICE-VALUE-TOTAL
               END-DISPLAY
           END-IF.


      *>****************************************************************
      *>   Rolls dice based on dice type inputted by user. Uses random
      *>   function based on current millisecond value to see the dice
      *>   roll.
      *>   Paragraph also will compute max values and sum totals of rolls.
      *>****************************************************************
       ROLL-DICE.

           COMPUTE DICE-VALUE = FUNCTION RANDOM * NUMBER-OF-SIDES + 1

           DISPLAY "Dice value: " DICE-VALUE
           ADD DICE-VALUE TO DICE-VALUE-TOTAL-NO-MOD

           IF DICE-VALUE-MAX-NO-MOD < DICE-VALUE
               MOVE DICE-VALUE TO DICE-VALUE-MAX-NO-MOD
           END-IF

           IF VALUE-MODIFIER NOT ZERO THEN

               ADD VALUE-MODIFIER TO DICE-VALUE

               IF DICE-VALUE-MAX < DICE-VALUE
                   MOVE DICE-VALUE TO DICE-VALUE-MAX
               END-IF

               DISPLAY "Dice Value (with modifier): " DICE-VALUE
               ADD DICE-VALUE TO DICE-VALUE-TOTAL
           END-IF.


      *>****************************************************************
      *>   Prints input instructions to the screen.
      *>****************************************************************
       PRINT-INSTRUCTIONS.
           DISPLAY SPACE
           DISPLAY "D&D Dice Roller v1.0"
           DISPLAY "By: Erik Eriksen"
           DISPLAY "Web: http://eriks.servehttp.com"
           DISPLAY "Build Date: 2021-03-12"
           DISPLAY SPACE

           DISPLAY
               "Enter dice roll in [number of dice]d[number of dice]+"
               "[modifier] format."
           END-DISPLAY

           DISPLAY SPACE
           DISPLAY "Example inputs:"
           DISPLAY
               "  3d08+4 to roll 3 8-sided dice with a +4 modifier "
               "value."
           END-DISPLAY

           DISPLAY
               "  1d20+1 to roll a 20 sided die with a +1 modifier "
               "value"
           END-DISPLAY

           DISPLAY
               "  2d4 to roll 2 4-sided dice with no modifier."
           END-DISPLAY

           DISPLAY SPACE
           DISPLAY
               "If adding a modifer, number of sides must contain "
               "leading zero" WS-NEW_LINE
               "on values less than 10 (ex: 06)"
           END-DISPLAY
           DISPLAY SPACE
           DISPLAY
               "If no modifier is present, number of sides do not "
               "require leading" WS-NEW_LINE
               "zero on values less than 10."
           END-DISPLAY

           DISPLAY SPACE
           DISPLAY "Enter 'q' to quit the application"

           DISPLAY SPACE.

       END PROGRAM DND-DICE-ROLLER-PROGRAM.
